'use strict';
/**
 * Start the real script.
 */
require(['jquery', 'AddNotificationFilterPreferenceLivetable', 'xwiki-bootstrap-switch', 'xwiki-events-bridge'], function ($, AddNotificationFilterPreferenceLivetable) {

  /**
   * Page initialization
   */
  $(function() {

    new AddNotificationFilterPreferenceLivetable($('#modal-add-filter-preference'), window['livetable_notificationFilterPreferencesLiveTable'], $('.filterPreferences button.btn-addfilter'),
      $('.filterPreferences').attr('data-user-url'));
    var initBootstrapSwitches = function () {
      $('#notificationFilterPreferencesLiveTable .notificationFilterPreferenceCheckbox, #notificationFilterPreferencesLiveTable .toggleableFilterPreferenceCheckbox').bootstrapSwitch({
        size: 'mini',
        labelText: '<span class=\"fa fa-bell\"><\/span>'
      });
    };
    $(document).on('xwiki:livetable:displayComplete', function (event, data) {
      // Enable bootstrap switches
      initBootstrapSwitches();
    });
    initBootstrapSwitches();

  });
});

define('AddNotificationFilterPreferenceLivetable', ['jquery', 'xwiki-meta', 'bootstrap'], function($, xm) {

  return function(modal, filterPreferencesLivetable, buttonAddFilter, userURL) {
    var self = this;
    self.modal = modal;
    self.filterPreferencesLivetable = filterPreferencesLivetable;
    self.buttonAddFilter = buttonAddFilter;
    self.userURL = userURL;
    self.filterTypeSelector = modal.find('select#notificationFilterTypeSelector');
    self.eventTypeSelector = modal.find('select#notificationFilterEventTypeSelector');
    self.notificationFormatSelector = modal.find('select#notificationFilterNotificationFormatSelector');
    self.filterScopeTree = modal.find('.location-tree');
    self.cancelButton = modal.find('button[data-action="cancel"]');
    self.submitButton = modal.find('button[data-action="submit"]');

    /**
     * Initialization
     */
    self.init = function () {
      self.buttonAddFilter.on('click', function() {
        self.filterScopeTree.xtree();
        self.submitButton.prop('disabled', 'disabled');

        self.filterScopeTree.on('changed.jstree', function(e, data) {
          if (data.selected.length > 0) {
            self.submitButton.prop('disabled', '');
          }
        });
      });

      self.submitButton.on('click', self.onSelectButtonClicked);
    };

    /**
     * On "submit" button clicked
     */
    self.onSelectButtonClicked = function() {
      self.modal.modal('toggle');

      var tree = $.jstree.reference(self.filterScopeTree);
      var nodes = tree.get_selected(true);

      var serviceReference = XWiki.Model.resolve('XWiki.Notifications.Code.NotificationPreferenceService', XWiki.EntityType.DOCUMENT);
      var serviceURL = new XWiki.Document(serviceReference).getURL('get', 'outputSyntax=plain');

      var eventTypesSelected = self.eventTypeSelector.val();
      var eventTypes;
      if (eventTypesSelected != null && eventTypesSelected.length > 0) {
        eventTypes = eventTypesSelected.join(',');
      }
      var params = {
        action: 'createScopeFilterPreference',
        target: 'user',
        user: self.modal.data('user'),
        filterType: self.filterTypeSelector.val(),
        filterFormats: self.notificationFormatSelector.val().join(','),
        eventTypes: eventTypes,
        csrf: xm.form_token,
        wiki: [],
        space: [],
        page: []
      }

      for (var i = 0; i < nodes.length; ++i) {
        var node = nodes[i];
        var reference = node.data.id;
        if (node.data.type == 'wiki') {
          params['wiki'].push(reference);
        } else if (node.data.type == 'document' && node.data.id.match('WebHome$')) {
          params['space'].push(reference.substring(0, reference.length - '.WebHome'.length));
        } else {
          params['page'].push(reference);
        }
      }

      // Saving
      var notification = new XWiki.widgets.Notification("Ihre Einstellungen werden gespeichert\u2026", 'inprogress');
      // We use a "traditional: true" argument here to be sure that the argument names are not transformed to wiki[] etc by jquery. For more info see doc about $.param.
      $.post({
        url: serviceURL,
        data: params,
        traditional: true
      }).then(data => {
        notification.hide();
        new XWiki.widgets.Notification("Einstellungen gespeichert!", 'done');

        // Reload the livetable
        self.filterPreferencesLivetable.refresh();
      }).catch(() => {
        notification.hide();
        new XWiki.widgets.Notification("Fehler beim Speichern der Einstellungen", 'error');
      });
    };

    // Call init()
    self.init();
  };

});

define('ToggleableFilterPreference', ['jquery', 'xwiki-bootstrap-switch'], function($) {
  /**
  * Construct the Toggleable Filter Preference object.
  */
  return function(domElement, userURL, row) {
    var self               = this;
    self.domElement        = $(domElement);
    self.userURL           = userURL;
    self.row               = row;
    /**
     * Initialization
     */
    self.init = function () {
      // On change
      self.domElement.on('switchChange.bootstrapSwitch', function(event, state) {
        setTimeout(function() { self.save(state); }, 1);
      });
    };

    /**
     * Save the filter preference
     */
    self.save = function(state) {
      var notification = new XWiki.widgets.Notification("Ihre Einstellungen werden gespeichert\u2026", 'inprogress');
      var filterName = self.domElement.attr('data-filterName');
      var objectNumber = self.domElement.attr('data-objectNumber');
      if (objectNumber == '') {
        // If the object does not exist yet, create it (make this change a minor edit so it's not displayed, by default, in notifications)
        var restURL = self.userURL + '/objects?media=json&minorRevision=true';
        var params = {
          'className': 'XWiki.Notifications.Code.ToggleableFilterPreferenceClass',
          'property#filterName': filterName,
          'property#isEnabled': state ? '1' : '0'
        };
        $.post(restURL, params).then(data => {
          self.domElement.attr('data-objectNumber', data.number);
          notification.hide();
          new XWiki.widgets.Notification("Filtereinstellung gespeichert!", 'done');
        }).catch(() => {
          notification.hide();
          new XWiki.widgets.Notification("Fehler beim Speichern der Filtereinstellung", 'error');
        });
      } else if (objectNumber != '') {
        // If the object already exist, just update its value (make this change a minor edit so it's not displayed, by default, in notifications)
        var restURL = self.userURL + '/objects/XWiki.Notifications.Code.ToggleableFilterPreferenceClass/' + objectNumber
          + '/properties/isEnabled?media=json&minorRevision=true';
        $.ajax(restURL, {
          method: 'PUT',
          contentType: 'text/plain',
          data: state ? '1' : '0'
        }).then(data => {
          notification.hide();
          new XWiki.widgets.Notification("Filtereinstellung gespeichert!", 'done');
          $(document).trigger('xwiki:livetable:notifications-filters:toggle', {'row': self.row, 'state': state});
        }).catch(() => {
          notification.hide();
          new XWiki.widgets.Notification("Fehler beim Speichern der Filtereinstellung", 'error');
        });
      }
    };

    // Call init
    self.init();
  };
});

require(['jquery', 'xwiki-meta', 'ToggleableFilterPreference', 'xwiki-bootstrap-switch', 'xwiki-events-bridge'], function ($, xm, ToggleableFilterPreference) {

  $(document).on('xwiki:livetable:notifications-filters:toggle', function(event, data) {
    var changedRow = $(data.row.isEnabled);
    if (changedRow.length > 0) {
      if (data.state) {
        changedRow.attr("checked", "checked");
      } else {
        changedRow.removeAttr("checked");
      }
      data.row.isEnabled = changedRow.get(0).outerHTML;
    }
  });

  // Globals
  var serviceReference = XWiki.Model.resolve('XWiki.Notifications.Code.NotificationPreferenceService', XWiki.EntityType.DOCUMENT);
  var serviceURL = new XWiki.Document(serviceReference).getURL('get', 'outputSyntax=plain');
  var userURL = $('.filterPreferences').attr('data-user-url');
  var targetUser = $('.filterPreferences').attr('data-user');

  // Callback on livetable row printing
  $(document).on('xwiki:livetable:notificationFilterPreferencesLiveTable:newrow', function(event, data) {
    // Delete action
    $(data.row).find('a.actiondelete').on('click', function(event) {
      event.preventDefault();
      new XWiki.widgets.ConfirmationBox({
        onYes: function() {
          var notif = new XWiki.widgets.Notification("Wird gel\u00F6scht\u2026", 'inprogress');
          $.post(serviceURL, {
            action: 'deleteFilterPreference',
            target: 'user',
            user: targetUser,
            filterPreferenceId: data.data.filterPreferenceId,
            csrf: xm.form_token
          }).then(() => {
            notif.hide();
            new XWiki.widgets.Notification("Filtereinstellung gel\u00F6scht!", 'done');
            window.livetable_notificationFilterPreferencesLiveTable.refresh();
          }).catch(() => {
            notif.hide();
            new XWiki.widgets.Notification("Fehler beim L\u00F6schen der Filtereinstellung", 'error');
          });
        }
      });
    });
    // Enable / Disable action for classic filter preferences
    $(data.row).find('.notificationFilterPreferenceCheckbox').on('switchChange.bootstrapSwitch', function(event, state) {
      // TODO: Translation keys
      var notif = new XWiki.widgets.Notification("Wird gespeichert\u2026", 'inprogress');
      $.post(serviceURL, {
        action: 'setFilterPreferenceEnabled',
        filterPreferenceId: data.data.filterPreferenceId,
        target: 'user',
        user: targetUser,
        enabled: state,
        csrf: xm.form_token
      }).then(() => {
        notif.hide();
        new XWiki.widgets.Notification("Filtereinstellung gespeichert!", 'done');
        $(document).trigger('xwiki:livetable:notifications-filters:toggle', {'row': data.data, 'state': state});
      }).catch(() => {
        notif.hide();
        new XWiki.widgets.Notification("Fehler beim Speichern der Filtereinstellung", 'error');
      });
    });
    // Enable / Disable action for toggleable filter preferences
    $(data.row).find('.toggleableFilterPreferenceCheckbox').each(function() {
      new ToggleableFilterPreference(this, userURL, data.data);
    });
  });

  // Initializer
  $(function() {
    // This script is usually loaded after the livetable is displayed and so the previous callback is not called.
    // Because of that, we need
    if (window.livetable_notificationFilterPreferencesLiveTable) {
      window.livetable_notificationFilterPreferencesLiveTable.refresh();
    }
  });
});
